package com.persagy.apm.translate.helper;

import com.persagy.apm.common.helper.SpringHelper;
import com.persagy.apm.translate.extension.BeanTransWrapperBuilder;
import com.persagy.apm.translate.extension.ITransWrapperBuilder;
import com.persagy.apm.translate.model.ITransable;
import com.persagy.apm.translate.model.Trans;
import com.persagy.apm.translate.model.TransMeta;
import com.persagy.apm.translate.service.ITranslateService;
import org.apache.commons.lang3.StringUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 数据翻译助手
 * @author Charlie Yu
 * @Date 2021-03-03
 */
public final class TranslatorHelper {

	/**
	 * 前台传到后台的翻译元数据
	 */
	public static final String TRANSOR_PARAM = "__transor";

	/**
	 * 线程级缓存
	 */
	private static ThreadLocal<List<TransMeta>> threaMetas = new ThreadLocal<>();

	/**
	 * 取得请求的翻译元数据
	 * @return 翻译元数据列表
	 */
	public static List<TransMeta> getMetas(){
		return threaMetas.get();
	}

    /**
     * 根据实体类创建翻译包装构建器
     * @param clazz　实体类
     * @return 翻译包装构建器
     */
	public static ITransWrapperBuilder createTransWrapperBuilder(Class<?> clazz){
		// 预留此方法，后续可增加扩展点
		return new BeanTransWrapperBuilder(clazz);
	}

	/**
	 * 设置请求的翻译元数据串
	 * @param transor 翻译描述信息
	 */
	public static void setTransor(String transor){
		if(StringUtils.isEmpty(transor)) {
            return;
        }
		List<TransMeta> metas = TransMeta.parseMetas(transor);
		if(metas != null) {
            threaMetas.set(metas);
        }
	}

	/**
	 * 通过注解放置翻译描述信息
	 * @param transes 注解数组
	 * @return 描述信息列表
	 */
	public static List<TransMeta> parseMetas(Trans[] transes) {
		List<TransMeta> metas = new ArrayList<>(transes.length);
		for(Trans trans: transes){
			TransMeta meta = new TransMeta();
			meta.setType(trans.type());
			meta.setParameter(trans.parameter());
			meta.setCondition(trans.condition());
			meta.setMapKey(trans.mapKey());
			meta.setBatch(trans.batch());
			meta.setResult(trans.result());
			meta.setTarget(trans.target());
			meta.setSource(trans.source());
			metas.add(meta);
		}
		return metas;
	}

	/**
	 * 通过注解放置翻译描述信息
	 * @param transes 注解数组
	 * @return 描述信息列表
	 */
	public static List<TransMeta> putMetas(Trans[] transes) {
		List<TransMeta> metas = parseMetas(transes);
		putMetas(metas);
		return metas;
	}

	/**
	 * 通过字符串取得翻译描述信息
	 * @param transor 翻译描述串json形式
	 * @return 描述信息列表
	 */
	public static List<TransMeta> putMetas(String transor) {
		if(StringUtils.isEmpty(transor)){
			return null;
		}
		List<TransMeta> metas =TransMeta.parseMetas(transor);
		if(metas != null) {
            putMetas(metas);
        }
		return metas;
	}
	/**
	 * 将翻译元数据放入线程变量中
	 * @param metas    翻译元数据列表
	 */
	public static void putMetas(List<TransMeta> metas){
		threaMetas.set(metas);
	}
	/**
	 * 清除缓存
	 */
	public static void clearMetas(){
		threaMetas.remove();
	}

	/**
	 * 翻译数据
	 * @param tansable 可翻译对象
	 * @param metas 元数据
	 * @return 翻译后数据
	 */
	@SuppressWarnings("unchecked")
	public static Object translate(ITransable tansable, List<TransMeta> metas){
		if(tansable == null || metas == null) {
            return tansable;
        }
		if(!tansable.isNeedTranslate()) {
            return tansable;
        }
		Object data = tansable.getTranslator();
		if(data == null) {
            return tansable;
        }
		if(tansable.isMapPackage() && data instanceof Map){
			Map<String,Object> map = (Map<String, Object>) data;
			for(Map.Entry<String, Object> en:map.entrySet()){
				Object child = translate(en.getValue(),metas);
				map.put(en.getKey(), child);
			}
		}else{
			data = translate(data,metas);
		}
		tansable.setTranslator(data);
		return tansable;
	}

	/**
	 * 翻译数据
	 * @param object 实体,实体列表,map或map列表
	 * @return 翻译后数据
	 */
	public static Object translate(Object object, List<TransMeta> transMeta) {
		if(object == null || transMeta == null || transMeta.size() == 0) {
            return object;
        }
		ITranslateService tranlate = SpringHelper.getBean(ITranslateService.class);
		return tranlate.translate(object, transMeta);
	}
	
	/**
	 * 翻译数据
	 * @param tansable 可翻译对象
	 * @return 翻译后数据
	 */
	public static Object translate(ITransable tansable){
		return translate(tansable,getMetas());
	}

	/**
	 * 翻译数据
	 * @param object 实体,实体列表,map或map列表
	 * @return 翻译后数据
	 */
	public static Object translate(Object object) {
		return translate(object,getMetas());
	}
	/**
	 * 翻译数据
	 * @param tansable 可翻译对象
	 * @param transor  翻译描述信息
	 * @return 翻译后数据
	 */
	public static Object translate(ITransable tansable,String transor){
		if(StringUtils.isEmpty(transor)){
			return tansable;
		}
		return translate(tansable,TransMeta.parseMetas(transor));
	}

	/**
	 * 翻译数据
	 * @param object 实体,实体列表,map或map列表
	 * @param transor    翻译描述信息
	 * @return 翻译后数据
	 */
	public static Object translate(Object object,String transor) {
		if(StringUtils.isEmpty(transor)){
			return object;
		}
		return translate(object,TransMeta.parseMetas(transor));
	}
}
